import 'reflect-metadata';
import { IJsonClassOptions } from '.';
import { Constructable, IMappingOptions, JsonSerializable, MappingParams } from './interfaces';
declare type JsonConstructor<T> = Constructable<T & JsonSerializable>;
/**
 * Decorator that auto-implements @see JsonSerializable interface.
 *
 * @export
 * @template T
 * @returns
 * @param ignoreMissingFields
 */
export declare function JsonClass<T>(options?: IJsonClassOptions): <C extends JsonConstructor<T>>(ctor: C) => C;
/**
 * Decorator for complex-type properties to be (de)serialized correctly.
 * Use this if the property is of a type that needs recursive (de)serialization.
 *
 * @export
 * @param {Constructable<any>} constructor the constructor type of the property.
 * @param name the name of the property
 * @returns the decorator for the property.
 */
export declare function JsonComplexProperty<T>(constructor: Constructable<T>, name?: string | null): (target: Object, propertyKey: string | symbol) => void;
/**
 * Decorator for complex-type array properties to be (de)serialized correctly.
 * Use this if the property is an array of a type that needs recursive (de)serialization.
 *
 * @export
 * @param {Constructable<any>} constructor the constructor type of the array items.
 * @param name the name of the property
 * @returns the decorator for the property.
 */
export declare function JsonArrayOfComplexProperty<T>(constructor: Constructable<T>, name?: string | null): (target: Object, propertyKey: string | symbol) => void;
/**
 * The basic decorator for array of simple properties. Required to enable (de)serialization of property.
 *
 * `params` has the same meaning that the one in @see JsonProperty .
 *
 * @export
 * @param {(string | MappingFn<any, any> | IMappingOptions<any, any>)} [params] the params
 * @returns the decorator for the property.
 */
export declare function JsonArray<T, R>(params?: MappingParams<T, R>): (target: Object, propertyKey: string | symbol) => void;
/**
 * The basic decorator for simple properties. Required to enable (de)serialization of property.
 *
 * `params` can be:
 * - a string, if only the name of source property is provided (`name` property of @see IMappingOptions ).
 * - a function, if only the mapping function of property is provided (`mappingFn` property of @see IMappingOptions ).
 * - an object compliant to @see IMappingOptions interface.
 *
 * @export
 * @param {(string | MappingFn<any, any> | IMappingOptions<any, any>)} [params] the params
 * @returns the decorator for the property.
 */
export declare function JsonProperty<T, R>(params?: MappingParams<T, R>): (target: Object, propertyKey: string | symbol) => void;
/**
 * A custom decorator factory function, in order to allow defining custom reusable decorators.
 *
 * @param serializeFn the function used for serializing the value.
 * @param deserializeFn the function used for deserializing the value.
 */
export declare function makeCustomDecorator<T>(serializeFn: (t: T) => any, deserializeFn: (arg: any) => T): (params?: string | IMappingOptions<any, T> | undefined) => (target: Object, propertyKey: string | symbol) => void;
/**
 * A custom decorator for handling objects as maps.
 *
 * @param params the mapping options to apply to the values of the map.
 */
export declare const JsonMap: (params?: string | IMappingOptions<any, any> | import("./interfaces").MappingFn<any, any> | undefined) => (target: Object, propertyKey: string | symbol) => void;
export {};
