import 'reflect-metadata';
import { Constructable, IMappingOptions, JsonSerializable } from './interfaces';
/**
 * Static class for JSON Mapping.
 *
 * @export
 * @class JsonMapper
 */
export declare class JsonMapper {
    /**
     * Serialization method.
     * Transform `val` into a new JSON value by applying the "serialization" step for each property decorator.
     *
     * It needs @see JsonSerializable implementation, and serializes only properties
     * decorated with some decorator from this library, or a custom one implemented using @see makeCustomDecorator.
     *
     * Annotated properties are serialized into a property using the `name` value as the destination name (defaults to the property name),
     * if the `serializeFn` is present, it is invoked to allow serialization customization.
     *
     * If `complexType` is specified, the property is treated as it had @see JsonComplexProperty decorator,
     * recursively calling @see JsonMapper.serialize .
     *
     * If `isArray` is specified, the property is treated as it is an array,
     * using respectively @see JsonArray or @see JsonArrayOfComplexProperty ,
     * according to other parameters.
     *
     * @static
     * @param {*} source the value to be serialized.
     * @returns {string} the transformed JSON value.
     * @memberof JsonMapper
     */
    static serialize(source: JsonSerializable): any;
    /**
     * Deserializes an array. @see deserialize
     * @static
     * @template T the type of output object
     * @param {Constructable<T>} ctor the destination constructor
     * @param jsonArray the array to be deserialized
     * @returns {T} the deserialized object
     * @memberof JsonMapper
     */
    static deserializeArray<T>(ctor: Constructable<T>, jsonArray: any[]): T[];
    /**
     * Deserialization method.
     * Deserialize `jsonObj` into an object built using `ctor`.
     *
     * It deserializes only properties
     * decorated with some decorator from this library, or a custom one implemented using @see makeCustomDecorator .
     *
     * Annotated properties are deserialized into a property using the `name` value as the source name (defaults to the property name),
     * if the `mappingFn` is present, it is invoked to allow deserialization customization.
     *
     * If `complexType` is specified, the property is treated as it had @see JsonComplexProperty decorator,
     * recursively calling @see JsonMapper.serialize .
     *
     * If `isArray` is specified, the property is treated as it is an array,
     * using respectively @see JsonArray or @see JsonArrayOfComplexProperty ,
     * according to other parameters.
     *
     * @static
     * @template T the type of output object
     * @param {Constructable<T>} ctor the destination constructor
     * @param {*} source the value to be deserialized
     * @returns {T} the deserialized object
     * @memberof JsonMapper
     */
    static deserialize<T>(ctor: Constructable<T>, source: any): T;
}
/**
 * Deserializes the input `val` property according to is decorator options.
 *
 * @param options the decorator options
 * @param valueToDeserialize the value to deserialize
 * @returns the deserialized value
 */
export declare function deserializeValue(options: IMappingOptions, valueToDeserialize: any): any;
/**
 * Serializes the input `val` property according to is decorator options.
 *
 * @param options the decorator options
 * @param valueToSerialize the value to serialize
 * @returns the serialized value
 */
export declare function serializeValue(options: IMappingOptions, valueToSerialize: any): any;
